@@ -26,25 +26,11 @@
 G_BEGIN_DECLS
 
 /**
- * GgitBlob:
+ * GgitAnnotatedCommit:
  *
- * Represents a blob object.
+ * Represents an annotated commit object.
  */
-typedef struct _GgitBlob GgitBlob;
-
-/**
- * GgitBlobOutputStream:
- *
- * Represents a blob stream object.
- */
-typedef struct _GgitBlobOutputStream GgitBlobOutputStream;
-
-/**
- * GgitBranch:
- *
- * Represents a branch object.
- */
-typedef struct _GgitBranch GgitBranch;
+typedef struct _GgitAnnotatedCommit GgitAnnotatedCommit;
 
 /**
  * GgitBranchEnumerator:
@@ -60,27 +46,6 @@ typedef struct _GgitBranchEnumerator GgitBranchEnumerator;
  */
 typedef struct _GgitCloneOptions GgitCloneOptions;
 
-/**
- * GgitCommit:
- *
- * Represents a commit object.
- */
-typedef struct _GgitCommit GgitCommit;
-
-/**
- * GgitCommitParents:
- *
- * Represents the parents of a commit object.
- */
-typedef struct _GgitCommitParents GgitCommitParents;
-
-/**
- * GgitConfig:
- *
- * Represents a git configuration.
- */
-typedef struct _GgitConfig GgitConfig;
-
 /**
  * GgitConfigEntry:
  *
@@ -89,25 +54,25 @@ typedef struct _GgitConfig GgitConfig;
 typedef struct _GgitConfigEntry GgitConfigEntry;
 
 /**
- * GgitCred:
+ * GgitCredSshInteractivePrompt:
  *
- * Represents a git credential.
+ * Represents a ssh interactive prompt.
  */
-typedef struct _GgitCred GgitCred;
+typedef struct _GgitCredSshInteractivePrompt GgitCredSshInteractivePrompt;
 
 /**
- * GgitCredPlaintext:
+ * GgitDiffBinary:
  *
- * Represents a plain text credential.
+ * Represents a diff binary.
  */
-typedef struct _GgitCredPlaintext GgitCredPlaintext;
+typedef struct _GgitDiffBinary GgitDiffBinary;
 
 /**
- * GgitDiff:
+ * GgitDiffBinaryFile:
  *
- * Represents a diff list.
+ * Represents a diff binary file.
  */
-typedef struct _GgitDiff GgitDiff;
+typedef struct _GgitDiffBinaryFile GgitDiffBinaryFile;
 
 /**
  * GgitDiffDelta:
@@ -137,13 +102,6 @@ typedef struct _GgitDiffHunk GgitDiffHunk;
  */
 typedef struct _GgitDiffLine GgitDiffLine;
 
-/**
- * GgitDiffOptions:
- *
- * Represents the options used when creating a #GgitDiff.
- */
-typedef struct _GgitDiffOptions GgitDiffOptions;
-
 /**
  * GgitDiffSimilarityMetric:
  *
@@ -151,13 +109,6 @@ typedef struct _GgitDiffOptions GgitDiffOptions;
  */
 typedef struct _GgitDiffSimilarityMetric GgitDiffSimilarityMetric;
 
-/**
- * GgitBlame:
- *
- * Represents a blame.
- */
-typedef struct _GgitBlame GgitBlame;
-
 /**
  * GgitBlameHunk:
  *
@@ -166,11 +117,11 @@ typedef struct _GgitBlame GgitBlame;
 typedef struct _GgitBlameHunk GgitBlameHunk;
 
 /**
- * GgitIndex:
+ * GgitFetchOptions:
  *
- * Represents an index object.
+ * Represents a git fetch options.
  */
-typedef struct _GgitIndex GgitIndex;
+typedef struct _GgitFetchOptions GgitFetchOptions;
 
 /**
  * GgitIndexEntries:
@@ -201,39 +152,18 @@ typedef struct _GgitIndexEntriesResolveUndo GgitIndexEntriesResolveUndo;
 typedef struct _GgitIndexEntryResolveUndo GgitIndexEntryResolveUndo;
 
 /**
- * GgitMergeTreeOptions:
+ * GgitMergeOptions:
  *
  * Represents the options used when merging.
  */
-typedef struct _GgitMergeTreeOptions GgitMergeTreeOptions;
+typedef struct _GgitMergeOptions GgitMergeOptions;
 
 /**
- * GgitNative:
+ * GgitNote:
  *
- * Represents a generic native object.
+ * Represents a note object.
  */
-typedef struct _GgitNative GgitNative;
-
-/**
- * GgitObject:
- *
- * Represents a generic object in a repository.
- */
-typedef struct _GgitObject GgitObject;
-
-/**
- * GgitObjectFactory:
- *
- * Represents an object factory.
- */
-typedef struct _GgitObjectFactory GgitObjectFactory;
-
-/**
- * GgitObjectFactoryBase:
- *
- * Represents the base type for objects created by an object factory.
- */
-typedef struct _GgitObjectFactoryBase GgitObjectFactoryBase;
+typedef struct _GgitNote GgitNote;
 
 /**
  * GgitOId:
@@ -250,11 +180,18 @@ typedef struct _GgitOId GgitOId;
 typedef struct _GgitPatch GgitPatch;
 
 /**
- * GgitPush:
+ * GgitRebaseOperation:
  *
- * Represents a git push.
+ * Represents a rebase operation.
  */
-typedef struct _GgitPush GgitPush;
+typedef struct _GgitRebaseOperation GgitRebaseOperation;
+
+/**
+ * GgitRebaseOptions:
+ *
+ * Represents the options used when rebasing.
+ */
+typedef struct _GgitRebaseOptions GgitRebaseOptions;
 
 /**
  * GgitRef:
@@ -284,13 +221,6 @@ typedef struct _GgitReflog GgitReflog;
  */
 typedef struct _GgitReflogEntry GgitReflogEntry;
 
-/**
- * GgitRemote:
- *
- * Represents a git remote.
- */
-typedef struct _GgitRemote GgitRemote;
-
 /**
  * GgitRemoteHead:
  *
@@ -299,11 +229,11 @@ typedef struct _GgitRemote GgitRemote;
 typedef struct _GgitRemoteHead GgitRemoteHead;
 
 /**
- * GgitRemoteCallbacks:
+ * GgitRemote:
  *
- * Represents a git remote callbacks.
+ * Represents a git remote.
  */
-typedef struct _GgitRemoteCallbacks GgitRemoteCallbacks;
+typedef struct _GgitRemote GgitRemote;
 
 /**
  * GgitRepository:
@@ -313,20 +243,6 @@ typedef struct _GgitRemoteCallbacks GgitRemoteCallbacks;
  */
 typedef struct _GgitRepository GgitRepository;
 
-/**
- * GgitRevisionWalker:
- *
- * Represents a revision walker.
- */
-typedef struct _GgitRevisionWalker GgitRevisionWalker;
-
-/**
- * GgitSignature:
- *
- * Represents an action signature.
- */
-typedef struct _GgitSignature GgitSignature;
-
 /**
  * GgitStatusOptions:
  *
@@ -341,13 +257,6 @@ typedef struct _GgitStatusOptions GgitStatusOptions;
  */
 typedef struct _GgitSubmodule GgitSubmodule;
 
-/**
- * GgitTag:
- *
- * Represents a tag object.
- */
-typedef struct _GgitTag GgitTag;
-
 /**
  * GgitTransferProgress:
  *
@@ -355,13 +264,6 @@ typedef struct _GgitTag GgitTag;
  */
 typedef struct _GgitTransferProgress GgitTransferProgress;
 
-/**
- * GgitTree:
- *
- * Represents a tree object.
- */
-typedef struct _GgitTree GgitTree;
-
 /**
  * GgitTreeEntry:
  *
@@ -369,13 +271,6 @@ typedef struct _GgitTree GgitTree;
  */
 typedef struct _GgitTreeEntry GgitTreeEntry;
 
-/**
- * GgitTreeBuilder:
- *
- * Represents a tree object.
- */
-typedef struct _GgitTreeBuilder GgitTreeBuilder;
-
 /**
  * GgitBlameOptions:
  *
@@ -383,6 +278,13 @@ typedef struct _GgitTreeBuilder GgitTreeBuilder;
  */
 typedef struct _GgitBlameOptions GgitBlameOptions;
 
+/**
+ * GgitRevertOptions:
+ *
+ * Represents the options used when reverting.
+ */
+typedef struct _GgitRevertOptions GgitRevertOptions;
+
 /**
  * GgitBranchType:
  * @GGIT_BRANCH_LOCAL: specifies a local branch.
@@ -398,10 +300,27 @@ typedef enum
 
 typedef enum
 {
-	GGIT_CAP_THREADS = 1 << 0,
-	GGIT_CAP_HTTPS   = 1 << 1
-} GgitCapFlags;
+	GGIT_FEATURE_THREADS = 1 << 0,
+	GGIT_FEATURE_HTTPS   = 1 << 1,
+	GGIT_FEATURE_SSH     = (1 << 2)
+} GgitFeatureFlags;
 
+/**
+ * GgitDiffBinaryType:
+ * @GIT_DIFF_BINARY_NONE: There is no binary delta.
+ * @GIT_DIFF_BINARY_LITERAL: The binary data is the literal contents of the file.
+ * @GIT_DIFF_BINARY_DELTA: The binary data is the delta from one side to the other.
+ *
+ * When producing a binary diff, the binary data returned will be
+ * either the deflated full ("literal") contents of the file, or
+ * the deflated binary delta between the two sides (whichever is
+ * smaller).
+ */
+typedef enum {
+	GGIT_DIFF_BINARY_NONE,
+	GGIT_DIFF_BINARY_LITERAL,
+	GGIT_DIFF_BINARY_DELTA
+} GgitDiffBinaryType;
 
 /**
  * GgitBlameFlags:
@@ -412,7 +331,7 @@ typedef enum
 typedef enum
 {
 	GGIT_BLAME_NORMAL                 = 0,
-	GGIT_BLAME_TRACK_COPIES_SAME_FILE = 1 << 0,
+	GGIT_BLAME_TRACK_COPIES_SAME_FILE = 1 << 0
 } GgitBlameFlags;
 
 /**
@@ -429,10 +348,10 @@ typedef enum
  */
 typedef enum
 {
-	GGIT_CONFIG_LEVEL_SYSTEM = 1,
-	GGIT_CONFIG_LEVEL_XDG = 2,
-	GGIT_CONFIG_LEVEL_GLOBAL = 3,
-	GGIT_CONFIG_LEVEL_LOCAL = 4,
+	GGIT_CONFIG_LEVEL_SYSTEM  = 1,
+	GGIT_CONFIG_LEVEL_XDG     = 2,
+	GGIT_CONFIG_LEVEL_GLOBAL  = 3,
+	GGIT_CONFIG_LEVEL_LOCAL   = 4,
 	GGIT_CONFIG_LEVEL_HIGHEST = -1
 } GgitConfigLevel;
 
@@ -519,6 +438,7 @@ typedef enum {
  * @GGIT_DIFF_SHOW_UNMODIFIED: show unmodified files.
  * @GGIT_DIFF_PATIENCE: generate using the "patience diff" algorithm.
  * @GGIT_DIFF_MINIMAL: take extra time to find minimal diff.
+ * @GGIT_DIFF_SHOW_BINARY: include deflate/delta information for binary files.
  *
  * How the diff should be generated.
  */
@@ -550,21 +470,22 @@ typedef enum {
 	GGIT_DIFF_SHOW_UNMODIFIED            = 1u << 26,
 
 	GGIT_DIFF_PATIENCE                   = 1u << 28,
-	GGIT_DIFF_MINIMAL                    = 1u << 29
+	GGIT_DIFF_MINIMAL                    = 1u << 29,
+	GGIT_DIFF_SHOW_BINARY                = 1u << 30
 } GgitDiffOption;
 
 /**
  * GgitDiffFlag:
  * @GGIT_DIFF_FLAG_BINARY: if the file is binary.
  * @GGIT_DIFF_FLAG_NOT_BINARY: if the file is not binary.
- * @GGIT_DIFF_FLAG_VALID_OID: if the OID is valid.
+ * @GGIT_DIFF_FLAG_VALID_ID: if the ID is valid.
  *
  * Describes the diff file and/or delta flags
  */
 typedef enum {
 	GGIT_DIFF_FLAG_BINARY     = 1 << 0,
 	GGIT_DIFF_FLAG_NOT_BINARY = 1 << 1,
-	GGIT_DIFF_FLAG_VALID_OID  = 1 << 2
+	GGIT_DIFF_FLAG_VALID_ID   = 1 << 2
 } GgitDiffFlag;
 
 /**
@@ -622,7 +543,7 @@ typedef enum {
 
 /* NOTE: keep in sync with git2/refs.h */
 typedef enum {
-	GGIT_FILE_MODE_NEW             = 0000000,
+	GGIT_FILE_MODE_UNREADABLE      = 0000000,
 	GGIT_FILE_MODE_TREE            = 0040000,
 	GGIT_FILE_MODE_BLOB            = 0100644,
 	GGIT_FILE_MODE_BLOB_EXECUTABLE = 0100755,
@@ -633,17 +554,50 @@ typedef enum {
 /* NOTE: keep in sync with git2/merge.h */
 /**
  * GgitMergeAutomergeMode:
- * @GGIT_MERGE_AUTOMERGE_MODE_NORMAL:
- * @GGIT_MERGE_AUTOMERGE_MODE_NONE:
- * @GGIT_MERGE_AUTOMERGE_MODE_FAVOR_OURS:
- * @GGIT_MERGE_AUTOMERGE_MODE_FAVOR_THEIRS:
+ * @GGIT_MERGE_FILE_FAVOR_NORMAL: When a region of a file is changed in both branches,
+ * a conflict will be recorded in the index so that `git_checkout` can produce
+ * a merge file with conflict markers in the working directory. This is the default.
+ * @GGIT_MERGE_FILE_FAVOR_OURS: When a region of a file is changed in both branches, the file
+ * created in the index will contain the "ours" side of any conflicting
+ * region. The index will not record a conflict.
+ * @GGIT_MERGE_FILE_FAVOR_THEIRS: When a region of a file is changed in both branches, the file
+ * created in the index will contain the "theirs" side of any conflicting
+ * region. The index will not record a conflict.
+ * @GGIT_MERGE_FILE_FAVOR_UNION: When a region of a file is changed in both branches, the file
+ * created in the index will contain each unique line from each side,
+ * which has the result of combining both files.  The index will not
+ * record a conflict.
  */
 typedef enum {
-	GGIT_MERGE_AUTOMERGE_MODE_NORMAL       = 0,
-	GGIT_MERGE_AUTOMERGE_MODE_NONE         = 1,
-	GGIT_MERGE_AUTOMERGE_MODE_FAVOR_OURS   = 2,
-	GGIT_MERGE_AUTOMERGE_MODE_FAVOR_THEIRS = 3
-} GgitMergeAutomergeMode;
+	GGIT_MERGE_FILE_FAVOR_NORMAL = 0,
+	GGIT_MERGE_FILE_FAVOR_OURS   = 1,
+	GGIT_MERGE_FILE_FAVOR_THEIRS = 2,
+	GGIT_MERGE_FILE_FAVOR_UNION  = 3
+} GgitMergeFileFavor;
+
+/**
+ * GgitMergeFileFlags:
+ * GGIT_MERGE_FILE_DEFAULT: Defaults.
+ * GGIT_MERGE_FILE_STYLE_MERGE: Create standard conflicted merge files.
+ * GGIT_MERGE_FILE_STYLE_DIFF3: Create diff3-style files.
+ * GGIT_MERGE_FILE_SIMPLIFY_ALNUM: Condense non-alphanumeric regions for simplified diff file.
+ * GGIT_MERGE_FILE_IGNORE_WHITESPACE: Ignore all whitespace.
+ * GGIT_MERGE_FILE_IGNORE_WHITESPACE_CHANGE: Ignore changes in amount of whitespace.
+ * GGIT_MERGE_FILE_IGNORE_WHITESPACE_EOL: Ignore whitespace at end of line.
+ * GGIT_MERGE_FILE_DIFF_PATIENCE: Use the "patience diff" algorithm.
+ * GGIT_MERGE_FILE_DIFF_MINIMAL: Take extra time to find minimal diff.
+ */
+typedef enum {
+	GGIT_MERGE_FILE_DEFAULT                  = 0,
+	GGIT_MERGE_FILE_STYLE_MERGE              = (1 << 0),
+	GGIT_MERGE_FILE_STYLE_DIFF3              = (1 << 1),
+	GGIT_MERGE_FILE_SIMPLIFY_ALNUM           = (1 << 2),
+	GGIT_MERGE_FILE_IGNORE_WHITESPACE        = (1 << 3),
+	GGIT_MERGE_FILE_IGNORE_WHITESPACE_CHANGE = (1 << 4),
+	GGIT_MERGE_FILE_IGNORE_WHITESPACE_EOL    = (1 << 5),
+	GGIT_MERGE_FILE_DIFF_PATIENCE            = (1 << 6),
+	GGIT_MERGE_FILE_DIFF_MINIMAL             = (1 << 7),
+} GgitMergeFileFlags;
 
 /**
  * GgitMergeTreeFlags:
@@ -653,17 +607,51 @@ typedef enum {
 	GGIT_MERGE_TREE_FIND_RENAMES = (1 << 0)
 } GgitMergeTreeFlags;
 
+/**
+ * GgitRebaseOperationType:
+ * @GGIT_REBASE_OPERATION_PICK: The given commit is to be cherry-picked.
+ * The client should commit the changes and continue if there are no conflicts.
+ * @GGIT_REBASE_OPERATION_REWORD: The given commit is to be cherry-picked,
+ * but the client should prompt the user to provide an updated commit message.
+ * @GGIT_REBASE_OPERATION_EDIT: The given commit is to be cherry-picked,
+ * but the client should stop to allow the user to edit the changes before
+ * committing them.
+ * @GGIT_REBASE_OPERATION_SQUASH: The given commit is to be squashed into
+ * the previous commit. The commit message will be merged with the previous message.
+ * @GGIT_REBASE_OPERATION_FIXUP: The given commit is to be squashed into the
+ * previous commit. The commit message from this commit will be discarded.
+ * @GGIT_REBASE_OPERATION_EXEC: No commit will be cherry-picked.
+ * The client should run the given command and (if successful) continue.
+ *
+ * Type of rebase operation in-progress.
+ */
+typedef enum {
+	GGIT_REBASE_OPERATION_PICK = 0,
+	GGIT_REBASE_OPERATION_REWORD,
+	GGIT_REBASE_OPERATION_EDIT,
+	GGIT_REBASE_OPERATION_SQUASH,
+	GGIT_REBASE_OPERATION_FIXUP,
+	GGIT_REBASE_OPERATION_EXEC
+} GgitRebaseOperationType;
+
 /* NOTE: keep in sync with git2/refs.h */
 /**
  * GgitRemoteDownloadTagsType:
- * @GGIT_REMOTE_DOWNLOAD_TAGS_AUTO:
- * @GGIT_REMOTE_DOWNLOAD_TAGS_NONE:
- * @GGIT_REMOTE_DOWNLOAD_TAGS_ALL:
+ * @GGIT_REMOTE_DOWNLOAD_TAGS_UNSPECIFIED: Use the setting from the configuration.
+ * @GGIT_REMOTE_DOWNLOAD_TAGS_AUTO: Ask the server for tags pointing to objects we're already
+ * downloading.
+ * @GGIT_REMOTE_DOWNLOAD_TAGS_NONE: Don't ask for any tags beyond the refspecs.
+ * @GGIT_REMOTE_DOWNLOAD_TAGS_ALL: Ask for the all the tags.
+ *
+ * Automatic tag following option
+ *
+ * Lets us select the --tags option to use.
  */
 typedef enum {
-	GGIT_REMOTE_DOWNLOAD_TAGS_AUTO = 0,
-	GGIT_REMOTE_DOWNLOAD_TAGS_NONE = 1,
-	GGIT_REMOTE_DOWNLOAD_TAGS_ALL  = 2
+	GGIT_REMOTE_DOWNLOAD_TAGS_UNSPECIFIED = 0,
+	GGIT_REMOTE_DOWNLOAD_TAGS_AUTO,
+	GGIT_REMOTE_DOWNLOAD_TAGS_NONE,
+	GGIT_REMOTE_DOWNLOAD_TAGS_ALL
 } GgitRemoteDownloadTagsType;
 
 /* NOTE: keep in sync with git2/remote.h */
@@ -678,7 +666,7 @@ typedef enum {
 typedef enum {
 	GGIT_REMOTE_COMPLETION_DOWNLOAD = 0,
 	GGIT_REMOTE_COMPLETION_INDEXING = 1,
-	GGIT_REMOTE_COMPLETION_ERROR = 2
+	GGIT_REMOTE_COMPLETION_ERROR    = 2
 } GgitRemoteCompletionType;
 
 /* NOTE: keep in sync with git2/refs.h */
@@ -786,7 +774,7 @@ typedef enum {
 
 /**
  * GgitSubmoduleIgnore:
- * @GGIT_SUBMODULE_IGNORE_RESET: reset to on-disk value.
+ * @GGIT_SUBMODULE_IGNORE_UNSPECIFIED: reset to on-disk value.
  * @GGIT_SUBMODULE_IGNORE_NONE: don't ignore any change.
  * @GGIT_SUBMODULE_IGNORE_UNTRACKED: ignore untracked files.
  * @GGIT_SUBMODULE_IGNORE_DIRTY: ignore changes in the working directory.
@@ -795,14 +783,30 @@ typedef enum {
  * Describes which submodules should be ignored.
  */
 typedef enum {
-	GGIT_SUBMODULE_IGNORE_RESET     = -1,
+	GGIT_SUBMODULE_IGNORE_UNSPECIFIED = -1,
 
-	GGIT_SUBMODULE_IGNORE_NONE      = 1,
+	GGIT_SUBMODULE_IGNORE_NONE = 1,
 	GGIT_SUBMODULE_IGNORE_UNTRACKED = 2,
-	GGIT_SUBMODULE_IGNORE_DIRTY     = 3,
-	GGIT_SUBMODULE_IGNORE_ALL       = 4,
+	GGIT_SUBMODULE_IGNORE_DIRTY = 3,
+	GGIT_SUBMODULE_IGNORE_ALL = 4
 } GgitSubmoduleIgnore;
 
+/**
+ * GgitSubmoduleRecurse:
+ * @GGIT_SUBMODULE_RECURSE_NO: do no recurse into submodules.
+ * @GGIT_SUBMODULE_RECURSE_YES: recurse into submodules.
+ * @GGIT_SUBMODULE_RECURSE_ONDEMAND: recurse into submodules only when
+ * commit not already in local clone.
+ *
+ * Options for submodule recurse.
+ * Represents the value of `submodule.$name.fetchRecurseSubmodules`
+ */
+typedef enum {
+	GGIT_SUBMODULE_RECURSE_NO = 0,
+	GGIT_SUBMODULE_RECURSE_YES = 1,
+	GGIT_SUBMODULE_RECURSE_ONDEMAND = 2
+} GgitSubmoduleRecurse;
+
 /**
  * GgitSubmoduleStatus:
  * FIXME
@@ -826,22 +830,24 @@ typedef enum {
 
 /**
  * GgitSubmoduleUpdate:
- * @GGIT_SUBMODULE_UPDATE_RESET: reset to on-disk value.
  * @GGIT_SUBMODULE_UPDATE_CHECKOUT: checkout the submodule.
  * @GGIT_SUBMODULE_UPDATE_REBASE: rebase the current branch onto the
  *                                commit recorded in the superproject.
  * @GGIT_SUBMODULE_UPDATE_MERGE: merge the commit recorded in the
  *                               superproject into the current branch.
  * @GGIT_SUBMODULE_UPDATE_NONE: do not update this submodule.
+ * @GGIT_SUBMODULE_UPDATE_DEFAULT: not used except as static initializer
+ * when we don't want any particular update rule to be specified.
  *
  * Describes how a submodule should be updated.
  */
 typedef enum {
-	GGIT_SUBMODULE_UPDATE_RESET    = -1,
 	GGIT_SUBMODULE_UPDATE_CHECKOUT = 1,
 	GGIT_SUBMODULE_UPDATE_REBASE   = 2,
 	GGIT_SUBMODULE_UPDATE_MERGE    = 3,
 	GGIT_SUBMODULE_UPDATE_NONE     = 4,
+
+	GGIT_SUBMODULE_UPDATE_DEFAULT = 0
 } GgitSubmoduleUpdate;
 
 /**
@@ -852,8 +858,8 @@ typedef enum {
  * Describes in which order a tree walk should be performed.
  */
 typedef enum {
-	GGIT_TREE_WALK_MODE_PRE = 0,
-	GGIT_TREE_WALK_MODE_POST = 1,
+	GGIT_TREE_WALK_MODE_PRE  = 0,
+	GGIT_TREE_WALK_MODE_POST = 1
 } GgitTreeWalkMode;
 
 /**
@@ -890,9 +896,7 @@ typedef enum {
 	GGIT_STATUS_OPTION_RENAMES_INDEX_TO_WORKDIR = (1 << 8),
 	GGIT_STATUS_OPTION_SORT_CASE_SENSITIVELY    = (1 << 9),
 	GGIT_STATUS_OPTION_SORT_CASE_INSENSITIVELY  = (1 << 10),
-	GGIT_STATUS_OPTION_DEFAULT                  = GGIT_STATUS_OPTION_INCLUDE_IGNORED |
-	                                              GGIT_STATUS_OPTION_INCLUDE_UNTRACKED |
-	                                              GGIT_STATUS_OPTION_RECURSE_UNTRACKED_DIRS
+	GGIT_STATUS_OPTION_DEFAULT                  = GGIT_STATUS_OPTION_INCLUDE_IGNORED | GGIT_STATUS_OPTION_INCLUDE_UNTRACKED | GGIT_STATUS_OPTION_RECURSE_UNTRACKED_DIRS
 } GgitStatusOption;
 
 /**
@@ -908,8 +912,8 @@ typedef enum {
  */
 typedef enum {
 	GGIT_STATUS_SHOW_INDEX_AND_WORKDIR = 0,
-	GGIT_STATUS_SHOW_INDEX_ONLY = 1,
-	GGIT_STATUS_SHOW_WORKDIR_ONLY = 2
+	GGIT_STATUS_SHOW_INDEX_ONLY        = 1,
+	GGIT_STATUS_SHOW_WORKDIR_ONLY      = 2
 } GgitStatusShow;
 
 /**
@@ -929,9 +933,107 @@ typedef enum
 	GGIT_ATTRIBUTE_CHECK_FILE_THEN_INDEX = 0,
 	GGIT_ATTRIBUTE_CHECK_INDEX_THEN_FILE = 1 << 0,
 	GGIT_ATTRIBUTE_CHECK_INDEX_ONLY      = 1 << 1,
-	GGIT_ATTRIBUTE_CHECK_NO_SYSTEM       = 1 << 2,
+	GGIT_ATTRIBUTE_CHECK_NO_SYSTEM       = 1 << 2
 } GgitAttributeCheckFlags;
 
+/**
+ * GgitPackbuilderStage:
+ * @GGIT_PACKBUILDER_STAGE_ADDING_OBJECTS: adding objects.
+ * @GGIT_PACKBUILDER_STAGE_DELTAFICATION: deltafication.
+ *
+ * Valid stages for pack building.
+ */
+typedef enum
+{
+	GGIT_PACKBUILDER_STAGE_ADDING_OBJECTS = 0,
+	GGIT_PACKBUILDER_STAGE_DELTAFICATION  = 1
+} GgitPackbuilderStage;
+
+typedef enum
+{
+	GGIT_CHECKOUT_NONE                    = 0,
+	GGIT_CHECKOUT_SAFE                    = (1u << 0),
+	GGIT_CHECKOUT_FORCE                   = (1u << 1),
+	GGIT_CHECKOUT_RECREATE_MISSING        = (1u << 2),
+	GGIT_CHECKOUT_ALLOW_CONFLICTS         = (1u << 4),
+	GGIT_CHECKOUT_REMOVE_UNTRACKED        = (1u << 5),
+	GGIT_CHECKOUT_REMOVE_IGNORED          = (1u << 6),
+	GGIT_CHECKOUT_UPDATE_ONLY             = (1u << 7),
+	GGIT_CHECKOUT_DONT_UPDATE_INDEX       = (1u << 8),
+	GGIT_CHECKOUT_NO_REFRESH              = (1u << 9),
+	GGIT_CHECKOUT_SKIP_UNMERGED           = (1u << 10),
+	GGIT_CHECKOUT_USE_OURS                = (1u << 11),
+	GGIT_CHECKOUT_USE_THEIRS              = (1u << 12),
+	GGIT_CHECKOUT_DISABLE_PATHSPEC_MATCH  = (1u << 13),
+	GGIT_CHECKOUT_SKIP_LOCKED_DIRECTORIES = (1u << 18),
+	GGIT_CHECKOUT_DONT_OVERWRITE_IGNORED  = (1u << 19),
+	GGIT_CHECKOUT_CONFLICT_STYLE_MERGE    = (1u << 20),
+	GGIT_CHECKOUT_CONFLICT_STYLE_DIFF3    = (1u << 21),
+	GGIT_CHECKOUT_DONT_REMOVE_EXISTING    = (1u << 22),
+	GGIT_CHECKOUT_DONT_WRITE_INDEX        = (1u << 23),
+	GGIT_CHECKOUT_UPDATE_SUBMODULES       = (1u << 16),
+	GGIT_CHECKOUT_UPDATE_SUBMODULES_IF_CHANGED = (1u << 17)
+} GgitCheckoutStrategy;
+
+typedef enum {
+	GGIT_CHECKOUT_NOTIFY_NONE      = 0,
+	GGIT_CHECKOUT_NOTIFY_CONFLICT  = (1u << 0),
+	GGIT_CHECKOUT_NOTIFY_DIRTY     = (1u << 1),
+	GGIT_CHECKOUT_NOTIFY_UPDATED   = (1u << 2),
+	GGIT_CHECKOUT_NOTIFY_UNTRACKED = (1u << 3),
+	GGIT_CHECKOUT_NOTIFY_IGNORED   = (1u << 4),
+
+	GGIT_CHECKOUT_NOTIFY_ALL       = 0x0FFFFu
+} GgitCheckoutNotifyFlags;
+
+typedef enum {
+	GGIT_DIFF_FIND_BY_CONFIG                  = 0,
+	GGIT_DIFF_FIND_RENAMES                    = (1u << 0),
+	GGIT_DIFF_FIND_RENAMES_FROM_REWRITES      = (1u << 1),
+	GGIT_DIFF_FIND_COPIES                     = (1u << 2),
+	GGIT_DIFF_FIND_COPIES_FROM_UNMODIFIED     = (1u << 3),
+	GGIT_DIFF_FIND_REWRITES                   = (1u << 4),
+	GGIT_DIFF_BREAK_REWRITES                  = (1u << 5),
+	GGIT_DIFF_FIND_AND_BREAK_REWRITES         = (GGIT_DIFF_FIND_REWRITES | GGIT_DIFF_BREAK_REWRITES),
+	GGIT_DIFF_FIND_FOR_UNTRACKED              = (1u << 6),
+	GGIT_DIFF_FIND_ALL                        = (0x0ff),
+	GGIT_DIFF_FIND_IGNORE_LEADING_WHITESPACE  = 0,
+	GGIT_DIFF_FIND_IGNORE_WHITESPACE          = (1u << 12),
+	GGIT_DIFF_FIND_DONT_IGNORE_WHITESPACE     = (1u << 13),
+	GGIT_DIFF_FIND_EXACT_MATCH_ONLY           = (1u << 14),
+	GGIT_DIFF_BREAK_REWRITES_FOR_RENAMES_ONLY = (1u << 15),
+	GGIT_DIFF_FIND_REMOVE_UNMODIFIED          = (1u << 16)
+} GgitDiffFindFlags;
+
+typedef enum
+{
+	GGIT_DIFF_FORMAT_EMAIL_NONE                         = 0,
+	GGIT_DIFF_FORMAT_EMAIL_EXCLUDE_SUBJECT_PATCH_MARKER = (1 << 0)
+} GgitDiffFormatEmailFlags;
+
+typedef enum
+{
+	GGIT_CREDTYPE_USERPASS_PLAINTEXT = (1u << 0),
+	GGIT_CREDTYPE_SSH_KEY            = (1u << 1),
+	GGIT_CREDTYPE_SSH_CUSTOM         = (1u << 2),
+	GGIT_CREDTYPE_DEFAULT            = (1u << 3),
+	GGIT_CREDTYPE_SSH_INTERACTIVE    = (1u << 4),
+} GgitCredtype;
+
+typedef enum
+{
+	GGIT_DIRECTION_FETCH = 0,
+	GGIT_DIRECTION_PUSH  = 1
+} GgitDirection;
+
+typedef enum
+{
+	GGIT_CLONE_LOCAL_AUTO     = 0,
+	GGIT_CLONE_LOCAL_LOCAL    = 1,
+	GGIT_CLONE_LOCAL_NO_LOCAL = 2,
+	GGIT_CLONE_LOCAL_NO_LINKS = 3
+} GgitCloneLocal;
+
 /**
  * GgitConfigCallback:
  * @entry: a #GgitConfigEntry.
@@ -963,21 +1065,18 @@ typedef gint (* GgitConfigMatchCallback) (GMatchInfo  *match_info,
                                           gpointer     user_data);
 
 /**
- * GgitCredAcquireCallback:
- * @url: the resource for which we are demanding a credential.
- * @username_from_url: (allow-none): The username that was embedded in a "user@host"
- *                                   remote url, or NULL if not included.
- * @allowed_types: a bitmask stating which cred types are OK to return.
- * @cred: (transfer full): newly created credential object.
+ * GgitDiffBinaryCallback:
+ * @delta: a #GgitDiffDelta.
+ * @binary: a #GgitDiffBinary.
  * @user_data: (closure): user-supplied data.
  *
- * Signature of a function which acquires a credential object.
+ * Called for each file.
+ *
+ * Returns: 0 to go continue or a #GgitError in case there was an error.
  */
-typedef gint (* GgitCredAcquireCallback) (const gchar *url,
-                                          const gchar *username_from_url,
-                                          guint        allowed_types,
-                                          GgitCred   **cred,
-                                          gpointer     user_data);
+typedef gint (* GgitDiffBinaryCallback) (GgitDiffDelta  *delta,
+                                         GgitDiffBinary *binary,
+                                         gpointer        user_data);
 
 /**
  * GgitDiffFileCallback:
@@ -1046,6 +1145,21 @@ typedef gint (* GgitDiffSimilarityMetricSimilarityCallback) (gint     *score,
                                                              gpointer  signature_b,
                                                              gpointer  user_data);
 
+/**
+ * GgitNoteCallback:
+ * @blob_id: id of the blob containing the message.
+ * @annotated_object_id: id of the git object being annotated.
+ * @user_data: (closure): user-suplied data.
+ *
+ * When iterating over all the notes, callback that will be
+ * issued per note. See ggit_repository_note_foreach().
+ *
+ * Returns: 0 to go continue or a #GgitError in case there was an error.
+ */
+typedef gint (* GgitNoteCallback) (GgitOId *blob_id,
+                                   GgitOId *annotated_object_id,
+                                   gpointer user_data);
+
 /**
  * GgitReferencesNameCallback:
  * @name: the name of the reference
@@ -1059,6 +1173,19 @@ typedef gint (* GgitDiffSimilarityMetricSimilarityCallback) (gint     *score,
 typedef gint (* GgitReferencesNameCallback) (const gchar *name,
                                              gpointer     user_data);
 
+/**
+ * GgitReferencesCallback:
+ * @reference: (transfer full): the reference.
+ * @user_data: (closure): user-supplied data.
+ *
+ * The type of the callback functions for retrieving the references
+ * in a #GgitRepository. See ggit_repository_references_foreach().
+ *
+ * Returns: 0 to go for the next references or a #GgitError in case there was an error.
+ */
+typedef gint (* GgitReferencesCallback) (GgitRef  *reference,
+                                         gpointer  user_data);
+
 /**
  * GgitRemoteListCallback:
  * @name: the name of the reference.
@@ -1094,7 +1221,7 @@ typedef gint (* GgitRemoteListCallback) (const gchar *name,
 typedef gint (* GgitStashCallback) (gsize        index,
                                     const gchar *message,
                                     GgitOId     *stash_oid,
-                                    gpointer    *user_data);
+                                    gpointer     user_data);
 
 /**
  * GgitStatusCallback:
@@ -1126,6 +1253,21 @@ typedef gint (* GgitSubmoduleCallback) (GgitSubmodule *submodule,
                                         const gchar   *name,
                                         gpointer       user_data);
 
+/**
+ * GgitTagCallback:
+ * @name: the tag name.
+ * @tag_oid: the tag oid.
+ * @user_data: (closure): user-suplied data.
+ *
+ * When iterating over all the tags, callback that will be
+ * issued per entry. See ggit_repository_tag_foreach().
+ *
+ * Returns: 0 to go continue or a #GgitError in case there was an error.
+ */
+typedef gint (* GgitTagCallback) (const gchar *name,
+                                  GgitOId     *tag_oid,
+                                  gpointer     user_data);
+
 /**
  * GgitTransferProgressCallback:
  * @stats: a #GgitTransferProgress.
